@@ -10,11 +10,11 @@ WWW::Datafinder - Perl API for Datafinder L<< http://datafinder.com >> API for m
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Carp qw(cluck);
 use Data::Dumper;
@@ -138,7 +138,9 @@ sub _cache_file_name {
     my ( $self, $query_params, $data ) = @_;
     
     my $md5 = md5_hex(Dumper($query_params).Dumper($data));
-    my $fname = catdir($self->cache_dir, substr($md5, 0, 2));
+    my $fname = catdir($self->cache_dir, 
+                       substr($md5, 0, 2),
+                       substr($md5, 2, 2));
     unless ( -d $fname ) {
         my $err;
         unless (
@@ -186,6 +188,7 @@ sub _transaction {
                 } else {
                     print "Retrieved ".Dumper($query_params).Dumper($data)." from cache $f\n" if $ENV{DEBUG};
                     $data->{cached} = $t;
+                    $data->{cache_object} = $f;
                     return $data;
                 }
             }
@@ -219,7 +222,7 @@ sub _transaction {
     # all is good, perhaps we should cache it?
     if ($res && $self->cache_time) {
 
-        unless ($res->{errros}) {
+        unless ($res->{erros}) {
             nstore($res, $f);
             print "Stored result in cache file $f\n" if $ENV{DEBUG};
         }
@@ -236,7 +239,7 @@ sub _transaction {
 
     my $csv = Text::CSV_XS->new;
     my $df  = WWW::Datafinder->new( {
-          api_key    => '456', # place real API key here
+          api_key    => '456', # place a real API key here
           cache_dir  => '/var/tmp/datafinder',
           cache_time => 3600 * 24 * 14
     }) or die 'Cannot create Datafinder object';
@@ -326,7 +329,7 @@ sub append_email {
     return $self->_transaction( $data, {} );
 }
 
-=head2 append_email( $data )
+=head2 append_phone( $data )
 
 Attempts to append customer's phone number based on his/her name and address
 Please see L<< https://datafinder.com/api/docs-demo >> for more
@@ -364,6 +367,45 @@ sub append_phone {
     return $self->_transaction( $data, {} );
 }
 
+=head2 append_demograph( $data )
+
+Attempts to append customer's demographic data on his/her name and address
+Please see L<< https://datafinder.com/api/docs-demo >> for more
+info regarding the parameter names and format of their values in C<$data>.
+Returns a reference to a hash, which contains the response
+received from the server.
+Returns C<undef> on failure, application then may call
+C<error_message()> method to get the detailed info about the error.
+
+    my $res = $df->append_demograph(
+        {
+            d_fulladdr => $cust->{Address},
+            d_city     => $cust->{City},
+            d_state    => $cust->{State},
+            d_zip      => $cust->{ZIP},
+            d_first    => $cust->{Name},
+            d_last     => $cust->{Surname}
+        }
+    );
+    if ( $res ) {
+        if ( $res->{'num-results'} ) {
+            # there is a match!
+            print "Got a match: " . Dumper( $res->{results} );
+        }
+    } else {
+        warn 'Something went wrong ' . $df->error_message();
+    }
+
+=cut
+
+sub append_demograph {
+    my ( $self, $data ) = @_;
+    $data->{service} = 'demograph';
+
+    return $self->_transaction( $data, {} );
+}
+
+
 =head2 error_message()
 
 Returns the detailed explanation of the last error. Empty string if
@@ -379,6 +421,23 @@ server - set environment variable DEBUG to a positive value.
 
 =cut
 
+=head1 CACHING
+
+The returned results can be cached in a local file, so the next time you
+want to retrieve the data for the same person, you get a faster response and
+do not have to spend money. Negative results (no match) are cached as well
+to speed things up.
+
+The hash key for a request is calculated as MD5 of all request parameters
+(including API key). The result is stored in a file (via Storable).
+Files are organized in the two-level directory structure (so
+fea5ffd3d65ee4d8bdf630677c0c5ff6.stor goes into /var/tmp/datafinder-cache/fe/a5/)
+to accommodate potentially large amount of files.
+
+When a result is returned from the cache instead of the real server, it has
+the C< cached > key set to the timestamp of the original data retrieval and
+C< cache_object > set to the filename of the cache file.
+
 =head1 AUTHOR
 
 Andrew Zhilenko, C<< <perl at putinhuylo.org> >>
